(function ($, Drupal) {
  'use strict';

  Drupal.behaviors.smartDate = {
    attach: function(context, settings) {
      //Create the label element
      var ad_label = $("<label>").text('All day ').addClass('allday-label');
      //Create the input element
      var ad_input = $('<input type="checkbox">').addClass('allday');
      //Insert the input into the label
      ad_input.prependTo(ad_label);

      $('.field--type-smartdate select').each(function(){
        setInitialDuration(this);
      }).parent().once('add-allday').append(ad_label);

      function setAllDay(element) {
        var checkbox = $(element);
        var wrapper = checkbox.parents('.fieldset-wrapper');
        var start_time = wrapper.find('.time-start input[type="time"]');
        var end_time = wrapper.find('.time-end input[type="time"]');
        var duration = wrapper.find('select');
        if (start_time.val() == '00:00:00' && end_time.val() == '23:59:00') {
          checkbox.prop('checked', true);
          checkbox.attr('data-duration', duration.attr('data-default'));
          start_time.fadeOut();
          end_time.fadeOut();
        }
        else {
          checkbox.attr('data-duration', duration.val());
        }
      }

      $('.allday').once('add-allday').each(function(){
        setAllDay(this);
      }).on('change', function(){
        checkAllDay(this);
      });

      // set the step value on time inputs
      $('.field--type-smartdate input[type="time"]').attr('step', '60');

      // update the time
      $('.field--type-smartdate .time-start input').on('change', function(){
        setEndDate(this);
      });

      // special handler for duration updates
      $('.field--type-smartdate select').on('change', function(){
        durationChanged(this);
      });

      // update the time
      $('.field--type-smartdate .time-end').on('change', function(){
        setDuration(this);
      });

      function setEndDate(element) {
        var wrapper = $(element).parents('.fieldset-wrapper');
        var duration_select = wrapper.find('select');
        if (duration_select.val() === 'custom') {
          var duration = parseInt(duration_select.attr('data-duration'));
        }
        else {
          var duration = parseInt(duration_select.val());
        }
        if (!duration || duration == 'custom') { return; }

        var start_time = wrapper.find('.time-start input[type="time"]').val();
        var start_date = wrapper.find('.time-start input[type="date"]').val();
        if (!start_time && start_date) {
          wrapper.find('.time-end input[type="date"]').val(start_date);
          return
        }

        var start_array = start_time.split(':');
        if (start_date.length) {
          var end = new Date(start_date);
        } else {
          var end = new Date();
        }
        end.setHours(start_array[0]);
        end.setMinutes(parseInt(start_array[1]) + duration);
        var end_val = pad(end.getHours(), 2) + ":" + pad(end.getMinutes(), 2);

        wrapper.find('.time-end input[type="time"]').val(end_val);
        end.setDate(end.getDate() + 1);
        var new_end = end.getFullYear() + '-' + pad(end.getMonth() + 1, 2) + '-' + pad(end.getDate(), 2);
        var end_string = wrapper.find('.time-end input[type="date"]').val(new_end);
      }

      function durationChanged(element) {
        if ($(element).val() === 'custom') {
          // reset end time and add focus
          var wrapper = $(element).parents('.fieldset-wrapper');
          var end_time = wrapper.find('.time-end input[type="time"]');
          end_time.val('').focus();
        }
        else {
          // fire normal setEndDate()
          setEndDate(element);
        }
      }

      function setInitialDuration(element) {
        var duration = $(element).val();
        if (duration == 'custom') {
          var wrapper = $(element).parents('.fieldset-wrapper');
          var duration = calcDuration(wrapper);
        }
        $(element).attr('data-duration', duration);
      }

      function setDuration(element) {
        var wrapper = $(element).parents('.fieldset-wrapper');
        var duration = calcDuration(wrapper);
        if (duration == 0) {
          return;
        }
        var duration_select = wrapper.find('select');
        duration_select.attr('data-duration', duration);
        if (duration_select.children('option[value="' + duration + '"]').length != 0){
          duration_select.val(duration);
        } else {
          duration_select.val('custom');
        }
      }

      function calcDuration(wrapper) {
        var start_time = wrapper.find('.time-start input[type="time"]').val();
        var start_date = wrapper.find('.time-start input[type="date"]').val();
        var end_time = wrapper.find('.time-end input[type="time"]').val();
        var end_date = wrapper.find('.time-end input[type="date"]').val();
        if (!start_time || !start_date || !end_time || !end_date) {
          return 0;
        }
        var start_array = start_time.split(':');
        var end_array = end_time.split(':');
        if (start_date !== end_date) {
          var start = new Date(start_date);
          start.setHours(start_array[0]);
          start.setMinutes(parseInt(start_array[1]));
          var end = new Date(end_date);
          end.setHours(end_array[0]);
          end.setMinutes(parseInt(end_array[1]));
          var duration = (end.getTime() - start.getTime()) / (60 * 1000);
        } else {
          var duration = (parseInt(end_array[0]) - parseInt(start_array[0])) * 60 + (parseInt(end_array[1]) - parseInt(start_array[1]));
        }
        return duration;
      }

      function checkAllDay(element) {
        var checkbox = $(element);
        var wrapper = checkbox.parents('.fieldset-wrapper');
        var start_time = wrapper.find('.time-start input[type="time"]');
        var end_time = wrapper.find('.time-end input[type="time"]');

        if (checkbox.is(':checked')) {
          // save the current start and end_date
          checkbox.attr('data-start', start_time.val());
          checkbox.attr('data-end', end_time.val());
          // set to all day $values
          start_time.fadeOut(0).val('00:00');
          end_time.fadeOut(0).val('23:59');
        }
        else {
          // restore from data $values
          if (checkbox.attr('data-start')) {
            start_time.val(checkbox.attr('data-start'));
          }
          if (checkbox.attr('data-end')) {
            end_time.val(checkbox.attr('data-end'));
          }
          else if (checkbox.attr('data-duration')) {
            var duration = wrapper.find('select');
            duration.val(checkbox.attr('data-duration'));
            duration.attr('data-duration', checkbox.attr('data-duration'));
            setEndDate(start_time);
          }
          start_time.fadeIn(0);
          end_time.fadeIn(0);
        }
      }

      function pad(str, max) {
        str = str.toString();
        return str.length < max ? pad("0" + str, max) : str;
      }
    }
  };
})(jQuery, Drupal);
